// =============================================================================
//
//   Terrain Texture Generator - TerrTexGen_MainForm.pas
//
// =============================================================================
//   Copyright  2002-2004 by Sascha Willems - http://www.delphigl.de
//   visit the Delphi OpenGL Community - http://www.delphigl.com
// =============================================================================
//   Contents of this file are subject to the GNU Public License (GPL) which can
//   be obtained here : http://opensource.org/licenses/gpl-license.php
// =============================================================================

unit TerrTexGen_MainForm;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  DateUtils,
  AppEvnts,
  ExtDlgs,
  ComCtrls,

  FastDIB,
  FastBlend,

  Textures,
  Geometry,
  dglOpenGL;

type
  TMainForm = class(TForm)
    TexInfoOpenDlg: TOpenDialog;
    TexInfoSaveDlg: TSaveDialog;
    SaveDialog2: TSaveDialog;
    OpenPictureDialog1: TOpenPictureDialog;
    ApplicationEvents1: TApplicationEvents;
    Timer1: TTimer;
    FPSTimer: TTimer;
    PageControl: TPageControl;
    TabSheet1: TTabSheet;
    Image1: TImage;
    Image2: TImage;
    Image3: TImage;
    Image4: TImage;
    Image5: TImage;
    Image6: TImage;
    ButtonGenerate: TButton;
    ProgressBar1: TProgressBar;
    StaticText1: TStaticText;
    ButtonLoad: TButton;
    ButtonSave: TButton;
    ButtonSize: TStaticText;
    TabSheet3: TTabSheet;
    Bevel1: TBevel;
    Panel1: TPanel;
    StaticText3: TStaticText;
    StaticText4: TStaticText;
    TabSheet2: TTabSheet;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    TabSheet4: TTabSheet;
    GroupBox1: TGroupBox;
    UseLighting: TCheckBox;
    LightDir: TScrollBar;
    Label8: TLabel;
    Label5: TLabel;
    SunPos: TScrollBar;
    Label6: TLabel;
    Label7: TLabel;
    TexInfo: TLabel;
    procedure ButtonSaveClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ButtonGenerateClick(Sender: TObject);
    procedure Image1Click(Sender: TObject);
    procedure Image2Click(Sender: TObject);
    procedure Image3Click(Sender: TObject);
    procedure Image4Click(Sender: TObject);
    procedure Image5Click(Sender: TObject);
    procedure ButtonLoadClick(Sender: TObject);
    procedure ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
    procedure TabSheet3Show(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormMouseWheelDown(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
    procedure FormMouseWheelUp(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
    procedure Image6Click(Sender: TObject);
    procedure ButtonSizeClick(Sender: TObject);
    procedure FPSTimerTimer(Sender: TObject);
    procedure StaticText4Click(Sender: TObject);
    procedure SunPosChange(Sender: TObject);
    procedure LightDirChange(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    procedure LoadTextureInfo(pFileName : String);
  end;

type
 THeightMap = object
   VertexArray   : array of array[0..2] of Single;
   TexCoordArray : array of array[0..1] of Single;
   HeightData    : array of array of Byte;
   Texture       : glUInt;
   procedure CreateFromBitmap(pFileName : String;pResolution : Integer);
  end;

var
  Tex         : array[0..4] of TBitmap;
  TexName     : array[0..4] of String;
  HeightMap   : THeightMap;
  LightMapTex : TFastDIB;
  TexDim      : Integer;
  TerrRes     : Integer;
  Frames      : Integer;
  FPS         : Single;
  MainForm    : TMainForm;
  Rot         : Single;
  Zoom        : Single;
  HMapName    : String;
  TexN        : String;
  RC          : HGLRC;
  DC          : HDC;
  MaxTexSize  : Integer;
  ShowPreview : Boolean;

implementation

{$R *.dfm}

// =============================================================================
//  THeightMap
// =============================================================================
procedure THeightMap.CreateFromBitmap(pFileName : String;pResolution : Integer);
type
 TVertexR = record
  x,y,z : Single;
  end;
var
 Vertex     : array[0..3] of TVertex;
 Bmp        : TBitMap;
 x,z,i      : LongInt;
 Resolution : Integer;
 Size       : Integer;
const
 VertPos : array[0..3] of array[0..1] of Byte = ((0,0),(0,1),(1,1),(1,0));
begin
SetLength(VertexArray,0);
SetLength(TexCoordArray,0);
Resolution := pResolution;
Size       := 32;
Bmp := TBitMap.Create;
try
 Bmp.LoadFromFile(pFileName);
 SetLength(HeightData, Resolution+1);
 for x := 0 to Resolution do
  begin
  SetLength(HeightData[x], Resolution+1);
  for z := 0 to Resolution do
   HeightData[x,z] := Trunc(Bmp.Canvas.Pixels[Trunc(X/Resolution*Bmp.Width),Trunc(Z/Resolution*Bmp.Height)]/ clWhite * 255);
  end;
finally
 Bmp.Free
end;
for x := 0 to Resolution-2 do
 for z := 0 to Resolution-2 do
  begin
  for i := 0 to 3 do
   begin
   Vertex[i,0] := (X + VertPos[I][0])/Resolution*Size-Size/2;
   Vertex[i,2] := (Z + VertPos[I][1])/Resolution*Size-Size/2;
   Vertex[i,1] := HeightData[X + VertPos[I][0],Z + VertPos[I][1]]*2;
   SetLength(VertexArray, Length(VertexArray)+1);
   VertexArray[High(VertexArray)][0] := Vertex[i,0];
   VertexArray[High(VertexArray)][1] := Vertex[i,1]*0.01;
   VertexArray[High(VertexArray)][2] := Vertex[i,2];
   SetLength(TexCoordArray, Length(TexCoordArray)+1);
   TexCoordArray[High(TexCoordArray)][0] := (Vertex[i,0]-Size/2)/Size;
   TexCoordArray[High(TexCoordArray)][1] := (-Vertex[i,2]-Size/2)/Size;
   end;
  end;
glVertexPointer(3, GL_FLOAT, 12, VertexArray);
glTexCoordPointer(2, GL_FLOAT, 8, TexCoordArray);
end;

// =============================================================================
//  GenerateShadows
// =============================================================================
//  Credits for (most) of this code should go to Delphic
// =============================================================================
procedure GenerateShadows(pHeightMapFileName, pOutPutFileName : String);
type
 TRay = record
  Origin, Direction : TVertex
  end;
 TNormalMap = array of array[0..2] of ShortInt;

// ==================================
//  CreateNormalMap
// ==================================
procedure CreateNormalMap(HM : TFastDIB;HeightScale : Single; var NormalMap : TNormalMap);
const
 VertexPoss : array[0..8] of array[0..1] of ShortInt = ((0,0),(1,-1),(1,0),(1,1),(0,1),(-1,1),(-1,0),(-1,-1),(0,-1));
 Faces      : array[0..7] of array[0..2] of Byte     = ((0,1,8),(0,2,1),(0,4,2),(4,3,2),(0,5,4),(0,6,5),(0,8,6),(8,7,6));
var
 Vertices          : array[0..8] of TVertex;
 Normals           : array[0..7] of TVertex;
 x,z               : LongInt;
 i                 : LongInt;
 VertexPosMultiply : LongInt;
 Normale           : TVertex;
 Smooth            : Integer;
begin
Smooth   := 2;
SetLength(NormalMap, HM.Width*HM.Height);
FillChar(Vertices[0,0], SizeOf(Vertices), 0);
for z := 0 to HM.Height - 1 do
 begin
 for x := 0 to HM.Width - 1 do  begin
  Normale[0] := 0;
  Normale[1] := 0;
  Normale[2] := 0;
  VertexPosMultiply := Smooth;
  for i := 0 to 8 do
   begin
   Vertices[I][0] := X + VertexPoss[I][0]*VertexPosMultiply;
   Vertices[I][2] := Z + VertexPoss[I][1]*VertexPosMultiply;
   if (Z + VertexPoss[I][1]*VertexPosMultiply>0) and (Z + VertexPoss[I][1]*VertexPosMultiply<HM.Width) and
      (X + VertexPoss[I][0]*VertexPosMultiply>0) and (X + VertexPoss[I][0]*VertexPosMultiply<HM.Height) then
   with HM.GetPixel(Z + VertexPoss[I][1]*VertexPosMultiply, X + VertexPoss[I][0]*VertexPosMultiply) do
    Vertices[I][1] := (R+G+B)/3*HeightScale/5;
   end;
  for i := 0 to 7 do
   begin
   Normals[I] := VectorCrossProduct(VectorNormalize(VectorSubtract(Vertices[Faces[I][0]],Vertices[Faces[I][1]])),
   VectorNormalize(VectorSubtract(Vertices[Faces[I][1]],Vertices[Faces[I][2]])));
   Normals[I] := VectorNormalize(Normals[I])
   end;
  for i := 0 to 7 do
   Normale := VectorAdd(Normale, Normals[I]);
  NormalizeVector(Normale);
  Normale                         := VectorScale(Normale,127.0);
  NormalMap[Z*(HM.Height) + X][0] := Trunc(Normale[0]);
  NormalMap[Z*(HM.Height) + X][1] := Trunc(Normale[1]);
  NormalMap[Z*(HM.Height) + X][2] := Trunc(Normale[2]);
  end;
 end;
end;
// ==================================
//  CreateLightMap
// ==================================
procedure CreateLightMap(HM : TFastDIB; HeightMapScale : Single;NormalMap : TNormalMap; Sun : TVertex; LightMap : TFastDIB; LightAmbient, LightDiffuse : Single; Raytracing : Boolean);
var
 HeightMapData : array of array of byte;
 Col           : TFColor;
// ==================================
//  PreCacheHeightMap
// ==================================
procedure PrecacheHeightmap;
var
 x,z : LongInt;
begin
SetLength(HeightMapData, HM.Width);
for x := 0 to Length(HeightMapData) -1 do
 SetLength(HeightMapData[X], HM.Height);
for z := 0 to HM.Height - 1 do
 for x := 0 to HM.Width - 1 do
  with HM.Pixels[z,x] do
   HeightMapData[X,Z] := Trunc((R+G+B)/3);
end;
// ==================================
//  RayIntersected
// ==================================
function RayIntersected(Ray : TRay; HM : TFastDIB; HeightMapScale : Single):Boolean;
var
 Position : TVertex;
 Vor      : TVertex;
begin
Result   := False;
Position := Ray.Origin;
Vor      := VectorNormalize(Ray.Direction);
Position := VectorAdd(Position, Vor);
while (Position[0] > 0) and (Position[0] < HM.Width - 1) and (Position[2] > 0) and (Position[2] < HM.Height - 1) do
 begin
 if HeightMapData[Trunc(Position[0]), Trunc(Position[2])]*HeightMapScale >= Position[1] then
  begin
  Result := True;
  Break
  end;
 Position := VectorAdd(Position, Vor)
 end;
end;
var
 x,z : LongInt;
 Ray : TRay;
 n   : TVertex;
 f   : Single;
begin
if Raytracing then
 PrecacheHeightmap;
Ray.Direction := VectorNormalize(Sun);
for z := 0 to HM.Height - 1 do
 begin
 for x := 0 to HM.Width - 1 do
  begin
  Ray.Origin[0] := X;
  with HM.GetPixel(z,x) do
   Ray.Origin[1] := (R+G+B)/3*HeightMapScale;
  Ray.Origin[2] := Z;
  if RayTracing and RayIntersected(Ray, HM, HeightMapScale) then
   f := LightAmbient
  else
   begin
   n[0] := NormalMap[Z*(HM.Height) + X][0];
   n[1] := NormalMap[Z*(HM.Height) + X][1];
   n[2] := NormalMap[Z*(HM.Height) + X][2];
   f    := LightAmbient+ LightDiffuse*(VectorDotProduct(VectorNormalize(n),VectorNormalize(Ray.Direction)));
   if f > 1.0 then
    f := 1.0;
   if f < 0.0 then
    f := 0.0
   end;
  Col := FRGB(Trunc(f*255), Trunc(f*255), Trunc(f*255));
  LightMap.SetPixel(z,x,Col);
  end
 end;
end;
var
 HeightMap      : TFastDIB;
 Sun            : TVertex;
 HeightMapScale : Single;
 NormalMap      : TNormalMap;
begin
HeightMap := TFastDIB.Create;
HeightMap.LoadFromFile(pHeightMapFileName);
LightMapTex := TFastDIB.Create;
Sun[0] := Cos(MainForm.LightDir.Position*Pi/180);
Sun[1] := MainForm.SunPos.Position/100;
Sun[2] := Sin(MainForm.LightDir.Position*Pi/180);
HeightMapScale := 1;
LightMapTex.SetSize(HeightMap.Width, HeightMap.Height, 24);
CreateNormalMap(HeightMap, HeightMapScale, NormalMap);
CreateLightMap(HeightMap, HeightMapScale, NormalMap, Sun, LightMapTex, 60/100, 50/100, True);
LightMapTex.SaveToFile('lm.bmp');
end;

// =============================================================================
//  LoadTextureInfo
// =============================================================================
procedure TMainForm.LoadTextureInfo(pFileName : String);
var
 F : TextFile;
 S : String;
 i : Integer;
begin
AssignFile(F, pFileName);
Reset(F);
for i := 0 to High(Tex) do
 begin
 ReadLn(F, S);
 if Tex[i] = nil then
  Tex[i] := TBitmap.Create;
 TexName[i] := S;
 Tex[i].LoadFromFile(S);
 end;
ReadLn(F, HMapName);
CloseFile(F);
Image2.Picture.LoadFromFile(TexName[0]);
Image3.Picture.LoadFromFile(TexName[1]);
Image4.Picture.LoadFromFile(TexName[2]);
Image5.Picture.LoadFromFile(TexName[3]);
Image6.Picture.LoadFromFile(TexName[4]);
if FileExists(HMapName) then
 Image1.Picture.LoadFromFile(HMapName)
else
 begin
 HMapName := 'terrain.bmp';
 Image1.Picture.LoadFromFile(HMapName);
 end
end;

// =============================================================================
//  Gibt den Prozentsatz einer Textur auf einer gegebenen Hhe zurck.
//  Wird bentigt, da die Texturen ber die Hhe langsam ineinander
//  bergehen.
// =============================================================================
function TexFactor(pH1, pH2, pStartHeight : Single) : Single;
begin
Result := (64-Abs(pH1-pH2))/64;
if Result > 1 then
 Result := 1;
if Result < 0 then
 Result := 0;
end;

// =============================================================================
//  Generiert aus der Hhenkarte die finale Terraintextur und sichert sie in
//  die Datei pOutPutFileName
// =============================================================================
procedure TexGen(pHeightMapFileName, pOutPutFileName : String);
var
 TexFact        : array[0..4] of Single;
 OldR,OldG,OldB : array[0..4] of Integer;
 SurfTex        : array of TFastDIB;
 TempFDIB       : TFastDIB;
 HMapTex        : TFastDIB;
 FinalTex       : TFastDIB;
 x,y,i          : Integer;
 HMapHeight     : Single;
 NCol           : TFColor;
 TmpCol         : record
                  r,g,b : Integer;
                  end;
begin
TexN := pOutPutFileName;
ChDir(ExtractFilePath(Application.ExeName));
FinalTex := TFastDIB.Create;
HMapTex  := TFastDIB.Create;
TempFDIB := TFastDIB.Create;
FinalTex.SetSize(TexDim, TexDim, 24);
HMapTex.SetSize(TexDim, TexDim, 24);
TempFDIB.LoadFromFile(pHeightMapFileName);
TempFDIB.StretchRect(HMapTex.hDC, 0, 0, TexDim, TexDim, 0, 0, TempFDIB.Width, TempFDIB.Height);
// Oberflchentexturen erstellen (ntig, falls Terraintextur grer als OberflchenTextur)
SetLength(SurfTex, Length(Tex));
for i := 0 to High(SurfTex) do
 begin
 SurfTex[i] := TFastDIB.Create;
 SurfTex[i].SetSize(TexDim, TexDim, 24);
 TempFDIB.LoadFromHandle(Tex[i].Handle);
 for y := 0 to SurfTex[i].Height div Tex[i].Height do
  for x := 0 to SurfTex[i].Width div Tex[i].Width do
   TempFDIB.Draw(SurfTex[i].hDC, x*Tex[i].Width, y*Tex[i].Height);
 end;
// Terraintextur generieren
for y := 0 to TexDim-1 do
 begin
 for x := 0 to TexDim-1 do
  begin
  HMapHeight := ((HMapTex.GetPixel(y,x).r+HMapTex.GetPixel(y,x).g+HMapTex.GetPixel(y,x).b) div 3);
  {TexFact[0] := TexFactor(StrToInt(Form1.H4End.Text), HMapHeight, StrToInt(Form1.H4Start.Text));
  TexFact[1] := TexFactor(StrToInt(Form1.H3End.Text), HMapHeight, StrToInt(Form1.H3Start.Text));
  TexFact[2] := TexFactor(StrToInt(Form1.H2End.Text), HMapHeight, StrToInt(Form1.H2Start.Text));
  TexFact[3] := TexFactor(StrToInt(Form1.H1End.Text), HMapHeight, StrToInt(Form1.H1Start.Text));
  TexFact[4] := TexFactor(StrToInt(Form1.H0End.Text), HMapHeight, StrToInt(Form1.H0Start.Text));}
  TexFact[0] := TexFactor(256, HMapHeight, 0);
  TexFact[1] := TexFactor(192, HMapHeight, 0);
  TexFact[2] := TexFactor(128, HMapHeight, 0);
  TexFact[3] := TexFactor(64, HMapHeight, 0);
  TexFact[4] := TexFactor(0, HMapHeight, 0);
  // Alle Texturenfarbwerte auslesen
  for i := 0 to High(Tex) do
   begin
   OldR[i] := SurfTex[i].GetPixel(y,x).r;
   OldG[i] := SurfTex[i].GetPixel(y,x).g;
   OldB[i] := SurfTex[i].GetPixel(y,x).b;
   end;
  // Neue RGB-Werte fr die Terraintextur generieren
  TmpCol.R := Round(TexFact[0]*OldR[0]) + Round(TexFact[1]*OldR[1]) + Round(TexFact[2]*OldR[2]) + Round(TexFact[3]*OldR[3]) + Round(TexFact[4]*OldR[4]);
  TmpCol.G := Round(TexFact[0]*OldG[0]) + Round(TexFact[1]*OldG[1]) + Round(TexFact[2]*OldG[2]) + Round(TexFact[3]*OldG[3]) + Round(TexFact[4]*OldG[4]);
  TmpCol.B := Round(TexFact[0]*OldB[0]) + Round(TexFact[1]*OldB[1]) + Round(TexFact[2]*OldB[2]) + Round(TexFact[3]*OldB[3]) + Round(TexFact[4]*OldB[4]);
  if TmpCol.R > 255 then TmpCol.R := 255;
  if TmpCol.G > 255 then TmpCol.G := 255;
  if TmpCol.B > 255 then TmpCol.B := 255;
  NCol.R := TmpCol.R;
  NCol.G := TmpCol.G;
  NCol.B := TmpCol.B;
  FinalTex.SetPixel(y,x,NCol);
  end;
 MainForm.ProgressBar1.Position := MainForm.ProgressBar1.Position+TexDim;
 end;
FinalTex.SaveToFile(pOutPutFileName);
HMapTex.Free;
FinalTex.Free;
TempFDIB.Free;
for i := 0 to High(SurfTex) do
 SurfTex[i].Free;
end;

// =============================================================================
//  TMainForm.ButtonSaveClick
// =============================================================================
procedure TMainForm.ButtonSaveClick(Sender: TObject);
var
 F : TextFile;
 i : Integer;
begin
if not TexInfoSaveDlg.Execute then
 exit;
AssignFile(F, TexInfoSaveDlg.FileName);
Rewrite(F);
for i := 0 to 4 do
 WriteLn(F, TexName[i]);
WriteLn(F, HMapName);
CloseFile(F);
end;

// =============================================================================
//  TMainForm.FormCreate
// =============================================================================
procedure TMainForm.FormCreate(Sender: TObject);
var
 p     : Integer;
begin
TexDim  := 256;
TerrRes := 128;
Zoom    := 1;
InitOpenGL;
DC := GetDC(Panel1.Handle);
RC := CreateRenderingContext(DC, [opDoubleBuffered], 32, 24, 0, 0, 0, 0);
ActivateRenderingContext(DC, RC);
glGetIntegerv(GL_DEPTH_BITS, @p);
glEnable(GL_TEXTURE_2D);
glEnable(GL_DEPTH_TEST);
glDepthFunc(GL_LESS);
glEnable(GL_NORMALIZE);
glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
glEnableClientState(GL_VERTEX_ARRAY);
glEnableClientState(GL_TEXTURE_COORD_ARRAY);
LoadTextureInfo('default.txt');
glGetIntegerv(GL_MAX_TEXTURE_SIZE, @MaxTexSize);
end;

// =============================================================================
//  TMainForm.ButtonGenerateClick
// =============================================================================
procedure TMainForm.ButtonGenerateClick(Sender: TObject);
var
 Time    : TDateTime;
 TerrTex : TFastDIB;
 NewLTex : TFastDIB;
 i       : Integer;
begin
if not FileExists(HMapName) then
 begin
 MessageDlg('Heightmap '+HMapName+' doesn''t exist!', mtError, [mbOK], 0);
 exit;
 end;
Time := Now;
ProgressBar1.Max := TexDim*TexDim;
if FileExists('terrain_tex.bmp') then
 begin
 i := 0;
 repeat
 if not FileExists('terrain_tex'+IntToStr(i)+'.bmp') then
  begin
  TexGen(HMapName, 'terrain_tex'+IntToStr(i)+'.bmp');
  break;
  end;
 inc(i);
 until i = 65535;
 end
else
 TexGen(HMapName, 'terrain_tex.bmp');
Image1.Picture.LoadFromFile(TexN);
TerrTex := TFastDIB.Create;
TerrTex.LoadFromFile(TexN);
if UseLighting.Checked then
 begin
 GenerateShadows(HMapName, TexN);
 if TerrTex.Width = LightMapTex.Width then
  MulBlend(TerrTex, TerrTex, LightMapTex)
 else
  begin
  NewLTex := TFastDIB.Create;
  NewLTex.SetSize(TerrTex.Width, TerrTex.Height, 24);
  LightMapTex.Stretch(NewLTex.hDC, 0, 0, NewLTex.Width, NewLTex.Height);
  MulBlend(TerrTex, TerrTex, NewLTex);
  NewLTex.Free;
  end;
 end;
TerrTex.SaveToFile(TexN);
TerrTex.Free;
StaticText1.Caption := TexN+' saved ('+IntToStr(MilliSecondsBetween(Now, Time))+'ms)';
ProgressBar1.Position := 0;
Image1.Picture.LoadFromFile(TexN);
if TexDim > MaxTexSize then
 begin
 TexInfo.Caption := 'Textur grer als Hardwarelimit! Keine Vorschau!';
 ShowPreview     := False;
 end
else
 begin
 TexInfo.Caption := '';
 ShowPreview     := True;
 end;
end;

// =============================================================================
//  TMainForm.Image1Click
// =============================================================================
procedure TMainForm.Image1Click(Sender: TObject);
begin
if OpenPictureDialog1.Execute then
 begin
 HMapName := OpenPictureDialog1.FileName;
 Image1.Picture.LoadFromFile(OpenPictureDialog1.FileName);
 end;
end;

// =============================================================================
//  TMainForm.Image2Click
// =============================================================================
procedure TMainForm.Image2Click(Sender: TObject);
begin
if OpenPictureDialog1.Execute then
 begin
 TexName[0] := OpenPictureDialog1.FileName;
 Tex[0].LoadFromFile(OpenPictureDialog1.FileName);
 Image2.Picture.LoadFromFile(OpenPictureDialog1.FileName);
 end;
end;

// =============================================================================
//  TMainForm.Image3Click
// =============================================================================
procedure TMainForm.Image3Click(Sender: TObject);
begin
if OpenPictureDialog1.Execute then
 begin
 TexName[1] := OpenPictureDialog1.FileName;
 Tex[1].LoadFromFile(OpenPictureDialog1.FileName);
 Image3.Picture.LoadFromFile(OpenPictureDialog1.FileName);
 end;
end;

// =============================================================================
//  TMainForm.Image4Click
// =============================================================================
procedure TMainForm.Image4Click(Sender: TObject);
begin
if OpenPictureDialog1.Execute then
 begin
 TexName[2] := OpenPictureDialog1.FileName;
 Tex[2].LoadFromFile(OpenPictureDialog1.FileName);
 Image4.Picture.LoadFromFile(OpenPictureDialog1.FileName);
 end;
end;

// =============================================================================
//  TMainForm.Image5Click
// =============================================================================
procedure TMainForm.Image5Click(Sender: TObject);
begin
if OpenPictureDialog1.Execute then
 begin
 TexName[3] := OpenPictureDialog1.FileName;
 Tex[3].LoadFromFile(OpenPictureDialog1.FileName);
 Image5.Picture.LoadFromFile(OpenPictureDialog1.FileName);
 end;
end;

// =============================================================================
//  TMainForm.Image6Click
// =============================================================================
procedure TMainForm.Image6Click(Sender: TObject);
begin
if OpenPictureDialog1.Execute then
 begin
 TexName[4] := OpenPictureDialog1.FileName;
 Tex[4].LoadFromFile(OpenPictureDialog1.FileName);
 Image6.Picture.LoadFromFile(OpenPictureDialog1.FileName);
 end;
end;

// =============================================================================
//  TMainForm.ButtonLoadClick
// =============================================================================
procedure TMainForm.ButtonLoadClick(Sender: TObject);
begin
if TexInfoOpenDlg.Execute then
 LoadTextureInfo(TexInfoOpenDlg.FileName);
end;

// =============================================================================
//  TMainForm.ApplicationEvents1Idle
// =============================================================================
procedure TMainForm.ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
begin
if not ShowPreview then
 exit;
if PageControl.TabIndex <> 2 then
 exit;
Done := False;
glEnable(GL_CULL_FACE);
glCullFace(GL_BACK);
glClearColor(0,0,0,1);
glViewport(0, 0,  Panel1.Width, Panel1.Height);
glMatrixMode(GL_PROJECTION);
glLoadIdentity;
gluPerspective(45, Panel1.Width/Panel1.Height, 0.1, 600);
glMatrixMode(GL_MODELVIEW);
glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
glLoadIdentity;
glTranslatef(0, 0, -50*Zoom);
glRotatef(30, 1, 0, 0);
glRotatef(Rot, 0, 1, 0);
glBindTexture(GL_TEXTURE_2D, HeightMap.Texture);
if Length(HeightMap.VertexArray) > 0 then
 glDrawArrays(GL_QUADS, 0, Length(HeightMap.VertexArray));
SwapBuffers(DC);
StaticText3.Caption := IntToStr(Length(HeightMap.VertexArray))+' Vertices / '+FloatToStr(FPS)+' fps';
inc(Frames);
end;

// =============================================================================
//  TMainForm.TabSheet3Show
// =============================================================================
procedure TMainForm.TabSheet3Show(Sender: TObject);
begin
if not ShowPreview then
 exit;
HeightMap.CreateFromBitmap(HMapName, TerrRes);
glDeleteTextures(1, @HeightMap.Texture);
if FileExists(TexN) then
 LoadTexture(TexN, HeightMap.Texture, False, GL_LINEAR, GL_LINEAR_MIPMAP_LINEAR, False)
else
 LoadTexture('terrain_tex.bmp', HeightMap.Texture, False, GL_LINEAR, GL_LINEAR_MIPMAP_LINEAR, FAlse);
end;

// =============================================================================
//  TMainForm.Timer1Timer
// =============================================================================
procedure TMainForm.Timer1Timer(Sender: TObject);
begin
Rot := Rot+0.25;
if Rot > 360 then
 Rot := 0;
end;

// =============================================================================
//  TMainForm.FormMouseWheelDown
// =============================================================================
procedure TMainForm.FormMouseWheelDown(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
begin
if Zoom < 2 then
 Zoom := Zoom+0.05;
end;

// =============================================================================
//  TMainForm.FormMouseWheelUp
// =============================================================================
procedure TMainForm.FormMouseWheelUp(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
begin
if Zoom > 0.1 then
 Zoom := Zoom-0.05;
end;

// =============================================================================
//  TMainForm.ButtonSizeClick
// =============================================================================
procedure TMainForm.ButtonSizeClick(Sender: TObject);
begin
case TexDim of
  256 : TexDim := 512;
  512 : TexDim := 1024;
 1024 : TexDim := 2048;
 2048 : TexDim := 4096;
 4096 : TexDim := 256;
end;
ButtonSize.Caption := IntToStr(TexDim)+'^2';
end;

// =============================================================================
//  TMainForm.FPSTimerTimer
// =============================================================================
procedure TMainForm.FPSTimerTimer(Sender: TObject);
begin
FPS := Frames;
Frames := 0;
end;

// =============================================================================
//  TMainForm.StaticText4Click
// =============================================================================
procedure TMainForm.StaticText4Click(Sender: TObject);
begin
TerrRes := TerrRes+32;
if TerrRes > 256 then
 TerrRes := 32;
StaticText4.Caption := 'Resolution : '+IntToStr(TerrRes);
HeightMap.CreateFromBitmap(HMapName, TerrRes);
end;

// =============================================================================
//  TMainForm.SunPosChange
// =============================================================================
procedure TMainForm.SunPosChange(Sender: TObject);
begin
Label6.Caption := FloatToStrF(SunPos.Position/100, ffFixed, 3, 3);
end;

// =============================================================================
//  TMainForm.LightDirChange
// =============================================================================
procedure TMainForm.LightDirChange(Sender: TObject);
begin
Label7.Caption := IntToStr(LightDir.Position);
end;

end.
